unit sGroupBox;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls, sPanel, sStyleUtil, sConst, sGraphUtils, sUtils;

type
  TsGroupBox = class(TsTransparentPanel)
  private
  protected
    FsStyle : TsPaintStyle;
    procedure DrawPanelBorders(Canvas: TCanvas);
    function TextHeight : integer;
    procedure WndProc (var Message: TMessage); override;
  public
    constructor Create (AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Paint; override;
  published
    property sStyle : TsPaintStyle read FsStyle write FsStyle;
  end;

implementation

{ TsGroupBox }

constructor TsGroupBox.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FsStyle := TsPaintStyle.Create(Self);
  FsStyle.COC := COC_TsGroupBox;
end;

destructor TsGroupBox.Destroy;
begin
  FreeAndNil(FsStyle);
  inherited Destroy;
end;

procedure TsGroupBox.DrawPanelBorders(Canvas: TCanvas);
var
  R: TRect;
begin
  GetWindowRect(Handle, R);
  OffsetRect(R, -R.Left, -R.Top);
  inc(R.Top, TextHeight div 2);

  if BevelOuter in [bvRaised] then begin
    PaintPanelBorder(Canvas.Handle, R.Left, R.Top, R.Right, R.Bottom, False, sStyle);
  end
  else if BevelOuter in [bvSpace, bvLowered] then begin
    PaintPanelBorder(Canvas.Handle, R.Left, R.Top, R.Right, R.Bottom, True, sStyle);
  end;
  InflateRect(R, - BorderWidth - BevelWidth, - BorderWidth - BevelWidth);

  if BevelInner in [bvRaised] then begin
    PaintPanelBorder(Canvas.Handle, R.Left, R.Top, R.Right, R.Bottom, False, sStyle);
  end
  else if BevelInner in [bvSpace, bvLowered] then begin
    PaintPanelBorder(Canvas.Handle, R.Left, R.Top, R.Right, R.Bottom, True, sStyle);
  end;

end;

procedure TsGroupBox.Paint;
var
  aRect: TRect;
  i, h: Integer;
  RStep, GStep, BStep: real;
  R, G, B: real;

begin
  if (csDestroying in ComponentState) then exit;
  if not sStyle.FDrawingStop then begin
    sStyle.FCacheBmp.Canvas.Font.Assign(Font);

    aRect := ClientRect;
    inc(aRect.Right);
    inc(aRect.Bottom);
    sStyle.FCacheBmp.Height := aRect.Bottom - aRect.Top;
    sStyle.FCacheBmp.Width := aRect.Right - aRect.Left;

    if sStyle.GetParentCache <> nil then begin
      FadeRect(sStyle.GetParentCache.Canvas, Rect(Left, Top, Left + Width, Top + TextHeight div 2), sStyle.FCacheBmp.Canvas, Point(0, 0), 100, clWhite, 0, ssRectangle);
      FadeRect(sStyle.GetParentCache.Canvas, Rect(Left, Top + TextHeight div 2, Left + Width, Top + Height), sStyle.FCacheBmp.Canvas, Point(0, TextHeight div 2), sStyle.Painting.Transparency, sStyle.GetActiveColor, 0, ssRectangle);
    end
    else begin
      Color := sStyle.GetActiveColor;
      sStyle.FCacheBmp.Canvas.Pen.Style := psClear;
      sStyle.FCacheBmp.Canvas.Brush.Style := bsSolid;
      sStyle.FCacheBmp.Canvas.Brush.Color := sStyle.GetActiveColor;
      sStyle.FCacheBmp.Canvas.Rectangle(aRect);
    end;

    DrawPanelBorders(sStyle.FCacheBmp.Canvas);

    if Caption <> '' then begin

      aRect.Top := 0;
      aRect.Bottom := aRect.Top + TextHeight;
      case Alignment of
        taLeftJustify : begin
          aRect.Left := 6;
          aRect.Right := aRect.Left + sStyle.FCacheBmp.Canvas.TextWidth(Caption) + 12;
        end;
        taRightJustify : begin
          aRect.Left := Width - sStyle.FCacheBmp.Canvas.TextWidth(Caption) - 12;
          aRect.Right := aRect.Left + sStyle.FCacheBmp.Canvas.TextWidth(Caption) + 12;
        end;
        taCenter : begin
          aRect.Left := (Width - sStyle.FCacheBmp.Canvas.TextWidth(Caption) - 12) div 2;
          aRect.Right := aRect.Left + sStyle.FCacheBmp.Canvas.TextWidth(Caption) + 12;
        end;
      end;

      if sStyle.GetParentCache <> nil then begin
        FadeRect(sStyle.GetParentCache.Canvas,
                 Rect(aRect.Left + Left,
                      aRect.Top + Top,
                      aRect.Right + Left,
                      aRect.Bottom + Top),
                 sStyle.FCacheBmp.Canvas, Point(aRect.Left, aRect.Top), sStyle.Painting.Transparency, sStyle.GetActiveColor, 0, ssRectangle);

      end
      else begin
        Color := sStyle.GetActiveColor;
        sStyle.FCacheBmp.Canvas.Pen.Style := psClear;
        sStyle.FCacheBmp.Canvas.Brush.Style := bsSolid;
        sStyle.FCacheBmp.Canvas.Brush.Color := sStyle.GetActiveColor;
        sStyle.FCacheBmp.Canvas.Rectangle(aRect);
      end;

      case sStyle.Painting.Transparency of
        1..99: begin
          sStyle.FCacheBmp.Canvas.Pen.Width := BevelWidth;
          if BevelOuter in [bvRaised] then begin
            sStyle.FCacheBmp.Canvas.Pen.Color := sStyle.Painting.ColorBorderTop;
          end
          else if BevelOuter in [bvSpace, bvLowered] then begin
            sStyle.FCacheBmp.Canvas.Pen.Color := sStyle.Painting.ColorBorderBottom;
          end;
          sStyle.FCacheBmp.Canvas.PolyLine([
                   Point(aRect.Left, aRect.Top + (TextHeight + BevelWidth) div 2),
                   Point(aRect.Left, aRect.Top + BevelWidth div 2),
                   Point(aRect.Right, aRect.Top + BevelWidth div 2),
                   Point(aRect.Right, aRect.Top + (TextHeight + BevelWidth) div 2)
                      ]);

          if BevelInner in [bvRaised] then begin
            sStyle.FCacheBmp.Canvas.Pen.Color := sStyle.Painting.ColorBorderTop;
          end
          else if BevelInner in [bvSpace, bvLowered] then begin
            sStyle.FCacheBmp.Canvas.Pen.Color := sStyle.Painting.ColorBorderBottom;
          end;
          sStyle.FCacheBmp.Canvas.PolyLine([
                   Point(aRect.Left, aRect.Top + BorderWidth + (TextHeight + BevelWidth) div 2),
                   Point(aRect.Left, Maxi(aRect.Bottom, Margin)),
                   Point(aRect.Right, Maxi(aRect.Bottom, Margin)),
                   Point(aRect.Right, aRect.Top + BorderWidth + (TextHeight + BevelWidth) div 2)
                      ]);
        end;
      end;

    end;

//    inc(aRect.Left, Margin + 6);
//    dec(aRect.Right, Margin + 6);
    aRect.Top := 0;
    aRect.Bottom := TextHeight;

    sStyle.WriteText(sStyle.FCacheBmp.Canvas, PChar(Caption), Enabled,
                            aRect.Left, aRect.Top, aRect.Right - aRect.Left, aRect.Bottom - aRect.Top,
                            DT_EXPANDTABS or DT_VCENTER or DT_CENTER);
//    WriteText(aRect, sStyle);

    sStyle.CopyFromCache(Canvas.Handle, 0, 0, Width, Height);
  end;
end;

function TsGroupBox.TextHeight: integer;
begin
  Result := Maxi(Margin, sStyle.FCacheBmp.Canvas.TextHeight('W')) + 6;
end;

procedure TsGroupBox.WndProc(var Message: TMessage);
begin
  if Assigned(FsStyle) then FsStyle.WndProc(Message);
  if Message.Result <> 1 then
    inherited;
end;

end.
