unit sShadowing;
(************************************************************************
sStyle components package 
Copyright (c) 2002 Serge Goncharov. All rights reserved.
                            Ukraine, Odessa
                            http://www.geocities.com/sstylish2002/
                            Serge@HyBce.com
 Version:	1.2, 24-02-2002
*************************************************************************)

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, sConst, sUtils;

type
  TsShadowing = class(TGraphicControl)
  private
    FOptimize : boolean;
    FShape : TsShadowingShape;
    FDrawingStop : boolean;
    procedure SetTransparency(const Value: integer);
    procedure SetOptimize(const Value: boolean);
    procedure SetShape(const Value: TsShadowingShape);
    procedure SetBlur(const Value: smallint);
  protected
    procedure WMEraseBkGND (var Message: TWMPaint); message WM_ERASEBKGND;
    procedure CopyFromCache(DC: longint; Left, Top, Right, Bottom: integer);
  public
    FBlur : smallint;
    FTransparency : integer;
    FCacheBmp : TBitmap;
    constructor Create(AOwner:TComponent); override;
    destructor Destroy; override;
    function GetParentCache : TBitmap;
    procedure Paint; override;
    procedure BeginDrawing;
    procedure EndDrawing;
  published
    property Transparency: integer read FTransparency write SetTransparency;
    property Optimize : boolean read FOptimize write SetOptimize;
    property Shape : TsShadowingShape read FShape write SetShape default ssRectangle;
    property Blur : smallint read FBlur write SetBlur default 1;
    property AutoSize;
    property Caption;
    property Align;
    property Anchors;
    property Color;
    property Top;
    property Left;
    property Height;
    property Width;
    property Constraints;
    property Visible;
    property OnClick;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
  end;

implementation

uses sPanel, sStyleUtil, sGraphUtils, sGroupBox;

{ TsShadowing }

constructor TsShadowing.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  ControlStyle := {ControlStyle + }[csReplicatable, csOpaque, csNoStdEvents];// - [csCaptureMouse];
  Color := clBlack;
  Height := 34;
  Width := 96;
  FShape := ssRectangle;
  FBlur := 1;
  FTransparency := 50;
  FCacheBmp := TBitMap.Create;
  FOptimize := True;
  ParentFont := True;
  AutoSize := True;
end;

procedure TsShadowing.BeginDrawing;
begin
  FDrawingStop := True;
end;

procedure TsShadowing.EndDrawing;
begin
  FDrawingStop := False;
  Invalidate;
end;

procedure TsShadowing.Paint;
var
  R: TRect;
  Flags: integer;
begin
  if (csDestroying in ComponentState) then exit;
  if not FDrawingStop then begin
    R := ClientRect;
    FCacheBmp.Height := R.Bottom - R.Top;
    FCacheBmp.Width := R.Right - R.Left;
    if FTransparency = 0 then begin
      FCacheBmp.Canvas.Brush.Color := Color;
      FCacheBmp.Canvas.FillRect(R);
    end
    else begin
      if GetParentCache <> nil then begin
        FadeRect(GetParentCache.Canvas, Rect(Left, Top, Left + Width, Top + Height), FCacheBmp.Canvas, Point(0, 0), FTransparency, Color, Blur, Shape);
      end
      else begin
        Caption := 'Put me on the sContainer, please.';
        FCacheBmp.Canvas.Brush.Color := clBlack;
        FCacheBmp.Canvas.Rectangle(R);
        FCacheBmp.Canvas.Brush.Style := bsClear;
        FCacheBmp.Canvas.Font := Font;
        FCacheBmp.Canvas.Font.Color := clWhite;

        R.Top := ((R.Bottom + R.Top) - FCacheBmp.Canvas.TextHeight('W')) div 2;
        R.Bottom := R.Top + FCacheBmp.Canvas.TextHeight('W');
        Flags := DT_EXPANDTABS or DT_VCENTER or DT_CENTER;
        Flags := DrawTextBiDiModeFlags(Flags);
        DrawText(FCacheBmp.Canvas.Handle, PChar(Caption), -1, R, Flags);
      end;
    end;
    Canvas.CopyRect(ClientRect, FCacheBmp.Canvas, ClientRect);
  end;
end;

procedure TsShadowing.SetBlur(const Value: smallint);
begin
  if FBlur <> Value then begin
    FBlur := Value;
    invalidate;
  End;
end;

procedure TsShadowing.SetOptimize(const Value: boolean);
begin
  FOptimize := Value;
  Invalidate;
end;

procedure TsShadowing.SetShape(const Value: TsShadowingShape);
begin
  if FShape <> Value then begin
    FShape := Value;
    Invalidate;
  end;
end;

procedure TsShadowing.SetTransparency(const Value: integer);
begin
  if FTransparency <> Value then begin
    if Value < 0 then FTransparency := 0
    else if Value > 100 then FTransparency := 100
    else FTransparency := Value;
    Invalidate;
  end;
end;

destructor TsShadowing.Destroy;
begin
  FCacheBmp.Free;
  inherited;
end;
(*
procedure TsShadowing.WMPaint(var Message: TWMPaint);
var
  SaveIndex: Integer;
  DC: HDC;
  PS: TPaintStruct;
  aRect : TRect;
  R: TRect;
  Flags: integer;
begin
//  if sStyle.FDrawingStop then Exit;

{
    if DC = 0 then begin
      Exit;
      Message.Result := 1;
    end;
}
    DC := Message.DC;
    if DC = 0 then begin
      DC := BeginPaint(Self.Handle, PS);
    end;
    SaveIndex := SaveDC(DC);

    R := ClientRect;
    FCacheBmp.Height := R.Bottom - R.Top;
    FCacheBmp.Width := R.Right - R.Left;
    if FTransparency = 0 then begin
      FCacheBmp.Canvas.Brush.Color := Color;
      FCacheBmp.Canvas.FillRect(R);
    end
    else begin
      if (Parent is TsCustomPanel) and Assigned(TsCustomPanel(Parent).sStyle.FCacheBmp) and Optimize then begin
        FadeRect(Self, TsCustomPanel(Parent).sStyle.FCacheBmp.Canvas, Rect(Left, Top, Left + Width, Top + Height), FCacheBmp.Canvas, Point(0, 0), FTransparency, Color, Blur, Shape);
      end
      else begin
        Caption := 'Put me on the sContainer, please.';
        FCacheBmp.Canvas.Brush.Color := clBlack;
        FCacheBmp.Canvas.Rectangle(R);
        FCacheBmp.Canvas.Brush.Style := bsClear;
        FCacheBmp.Canvas.Font := Font;
        FCacheBmp.Canvas.Font.Color := clWhite;

        R.Top := ((R.Bottom + R.Top) - FCacheBmp.Canvas.TextHeight('W')) div 2;
        R.Bottom := R.Top + FCacheBmp.Canvas.TextHeight('W');
        Flags := DT_EXPANDTABS or DT_VCENTER or DT_CENTER;
        Flags := DrawTextBiDiModeFlags(Flags);
        DrawText(FCacheBmp.Canvas.Handle, PChar(Caption), -1, R, Flags);
      end;
    end;

    if Message.DC = 0 then begin
      aRect := PS.rcPaint;
    end
    else begin
      aRect := ClientRect;
    end;
    CopyFromCache(DC, aRect.Left, aRect.Top, aRect.Right, aRect.Bottom);
    RestoreDC(DC, SaveIndex);
    if Message.DC = 0 then EndPaint(Handle, PS);
    Message.Result := 1;
//  end
//  else Inherited;
end;
*)
procedure TsShadowing.CopyFromCache(DC, Left, Top, Right, Bottom: integer);
begin
  BitBlt(DC, Left, Top, Right, Bottom, FCacheBmp.Canvas.Handle, Left, Top, SRCCOPY);
end;

procedure TsShadowing.WMEraseBkGND(var Message: TWMPaint);
begin
  Message.Result := 1;
end;

function TsShadowing.GetParentCache: TBitmap;
begin
  if Parent is TsCustomPanel then Result := TsCustomPanel(Parent).sStyle.FCacheBmp
  else if Parent is TsGroupBox then Result := TsGroupBox(Parent).sStyle.FCacheBmp
  else Result := nil;
end;

end.
